#include <FastLED.h>

 FASTLED_USING_NAMESPACE

// FastLED "100-lines-of-code" demo reel, showing just a few 
// of the kinds of animation patterns you can quickly and easily 
// compose using FastLED.  
//
// This example also shows one easy way to define multiple 
// animations patterns and have them automatically rotate.
//
// -Mark Kriegsman, December 2014
//
// The code has been modified By Kevin Fischer for the LED Hexagon art project in June 2024.

#if defined(FASTLED_VERSION) && (FASTLED_VERSION < 3001000)
#warning "Requires FastLED 3.1 or later; check github for latest code."
#endif

#define DATA_PIN    2 // LED strip data pin on Nano
//#define CLK_PIN   4
#define LED_TYPE    WS2812  // for our needs, this works for the WS2811 devices too.
#define COLOR_ORDER GRB
#define NUM_LEDS    37
CRGB leds[NUM_LEDS];

int BRIGHTNESS = 16;
#define FRAMES_PER_SECOND  60
const int photores_analogInPin = A2;  // Analog input pin
int photores_Value = 0;        //
int photores_SCALED_Value = 0;        //



void setup() {
  delay(3000); // 3 second delay for recovery
       // initialize serial communications at 9600 bps:
  Serial.begin(9600); 
   // read the analog in value of photores:
  photores_Value = analogRead(photores_analogInPin);
  // map it to the range of the analog out:
  photores_SCALED_Value = map(photores_Value, 0, 1023, 0, 255);

  // print the results to the Serial Monitor:
  Serial.print("Analog_Read = ");
  Serial.print(photores_Value);
  Serial.print("\t Scaled = ");
  Serial.println(photores_SCALED_Value);
  if (photores_SCALED_Value < 8) {photores_SCALED_Value = 8;}
  if (photores_SCALED_Value > 127) {photores_SCALED_Value = 127;}
  Serial.print("\t Adjusted = ");
  Serial.println(photores_SCALED_Value);
int BRIGHTNESS = photores_SCALED_Value;   // DEFINE BRIGHTNESS BASED ON SENSOR VALUE
  
  // tell FastLED about the LED strip configuration
  FastLED.addLeds<LED_TYPE,DATA_PIN,COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
  //FastLED.addLeds<LED_TYPE,DATA_PIN,CLK_PIN,COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);

  // set master brightness control
  FastLED.setBrightness(BRIGHTNESS);
}


// List of patterns to cycle through.  Each is defined as a separate function below.
typedef void (*SimplePatternList[])();
SimplePatternList gPatterns = { rainbow, rainbowWithGlitter, confetti, sinelon, juggle, bpm };

uint8_t gCurrentPatternNumber = 0; // Index number of which pattern is current
uint8_t gHue = 0; // rotating "base color" used by many of the patterns
  
void loop()
{
  // Call the current pattern function once, updating the 'leds' array
  gPatterns[gCurrentPatternNumber]();

  // send the 'leds' array out to the actual LED strip
  FastLED.show();  
  // insert a delay to keep the framerate modest
  FastLED.delay(3000/FRAMES_PER_SECOND); 

  // do some periodic updates
  EVERY_N_MILLISECONDS( 20 ) { gHue++; } // slowly cycle the "base color" through the rainbow
  EVERY_N_SECONDS( 30 ) { nextPattern(); } // change patterns periodically
  EVERY_N_MILLISECONDS( 250 ) { updateBrightness(); } // update brightness every 250 millisseconds
}

#define ARRAY_SIZE(A) (sizeof(A) / sizeof((A)[0]))

void  nextPattern()
{
  // add one to the current pattern number, and wrap around at the end
  gCurrentPatternNumber = (gCurrentPatternNumber + 1) % ARRAY_SIZE( gPatterns);
}

void rainbow()
{
  // FastLED's built-in rainbow generator
  fill_rainbow( leds, NUM_LEDS, gHue, 5);
  fadeToBlackBy( leds, NUM_LEDS, 10);
}

void rainbowWithGlitter() 
{
  // built-in FastLED rainbow, plus some random sparkly glitter
  rainbow();
  addGlitter(200);  // lots more glitter (was 20?)
  fadeToBlackBy( leds, NUM_LEDS, 10);
}

void addGlitter( fract8 chanceOfGlitter) 
{
  if( random8() < chanceOfGlitter) {
    leds[ random16(NUM_LEDS) ] += CRGB::Red; // red sparkle
  }
}

void confetti() 
{
  // random colored speckles that blink in and fade smoothly
  fadeToBlackBy( leds, NUM_LEDS, 10);
  int pos = random16(NUM_LEDS);
  leds[pos] += CHSV( gHue + random8(64), 200, 255);
  
}

void sinelon()
{
  // a colored dot sweeping back and forth, with fading trails
  fadeToBlackBy( leds, NUM_LEDS, 5);
  int pos = beatsin16( 13, 0, NUM_LEDS-1 );
  leds[pos] += CHSV( gHue, 255, 192);
}

void bpm()
{
  // colored stripes pulsing at a defined Beats-Per-Minute (BPM)
  uint8_t BeatsPerMinute = 10;  // original value was 62
  CRGBPalette16 palette = PartyColors_p;
  uint8_t beat = beatsin8( BeatsPerMinute, 64, 255);
  for( int i = 0; i < NUM_LEDS; i++) { //9948
    leds[i] = ColorFromPalette(palette, gHue+(i*2), beat-gHue+(i*10));
  }
}

void juggle() {
  // eight colored dots, weaving in and out of sync with each other
  fadeToBlackBy( leds, NUM_LEDS, 20);
  byte dothue = 0;
  for( int i = 0; i < 8; i++) {
    leds[beatsin16( i+7, 0, NUM_LEDS-1 )] |= CHSV(dothue, 200, 255);
    dothue += 32;
  }
}

void updateBrightness() {
  // read the analog pin and updated the brightness
   // read the analog in value of photores:
  photores_Value = analogRead(photores_analogInPin);
  // map it to the range of the analog out:
  photores_SCALED_Value = map(photores_Value, 0, 1023, 0, 255);
  if (photores_SCALED_Value < 4) {photores_SCALED_Value = 4;} //set minimum brightness level
  if (photores_SCALED_Value > 127) {photores_SCALED_Value = 127;} //set maximum brightness level
  int BRIGHTNESS = photores_SCALED_Value;   // DEFINE BRIGHTNESS BASED ON SENSOR VALUE
  FastLED.setBrightness(BRIGHTNESS);

}
